@@ -241,9 +241,9 @@ vips_min_build( VipsObject *object )
 			"y_array", y_array,
 			NULL );
 
-		vips_area_unref( (VipsArea *) out_array );
-		vips_area_unref( (VipsArea *) x_array );
-		vips_area_unref( (VipsArea *) y_array );
+		vips_area_unref( VIPS_AREA( out_array ) );
+		vips_area_unref( VIPS_AREA( x_array ) );
+		vips_area_unref( VIPS_AREA( y_array ) );
 	}
 
 #ifdef DEBUG
@@ -492,9 +492,9 @@ vips_min_init( VipsMin *min )
 }
 
 /**
- * vips_min:
+ * vips_min: (method)
  * @in: input #VipsImage
- * @out: output pixel minimum
+ * @out: (out): output pixel minimum
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
