@@ -261,7 +261,7 @@ vips_colour_attach_profile( VipsImage *im, const char *filename )
 	char *data;
 	size_t data_length;
 
-	if( !(data = vips__file_read_name( filename, VIPS_ICC_DIR, 
+	if( !(data = vips__file_read_name( filename, vips__icc_dir(), 
 		&data_length )) ) 
 		return( -1 );
 	vips_image_set_blob( im, VIPS_META_ICC_NAME, 
@@ -752,7 +752,7 @@ vips_colour_operation_init( void )
 	extern GType vips_scRGB2BW_get_type( void ); 
 	extern GType vips_XYZ2scRGB_get_type( void ); 
 	extern GType vips_scRGB2sRGB_get_type( void ); 
-#if defined(HAVE_LCMS) || defined(HAVE_LCMS2)
+#ifdef HAVE_LCMS2
 	extern GType vips_icc_import_get_type( void ); 
 	extern GType vips_icc_export_get_type( void ); 
 	extern GType vips_icc_transform_get_type( void ); 
@@ -786,7 +786,7 @@ vips_colour_operation_init( void )
 	vips_HSV2sRGB_get_type(); 
 	vips_XYZ2scRGB_get_type();
 	vips_scRGB2sRGB_get_type();
-#if defined(HAVE_LCMS) || defined(HAVE_LCMS2)
+#ifdef HAVE_LCMS2
 	vips_icc_import_get_type();
 	vips_icc_export_get_type();
 	vips_icc_transform_get_type();
