@@ -540,9 +540,33 @@ vips_image_expected_sig( VipsImage *image )
 		expected_sig = cmsSigYxyData;
 		break;
 
+	case VIPS_INTERPRETATION_MULTIBAND:
+		/* A generic many-band image. Try to guess from the number of 
+		 * image bands instead.
+		 */
+		switch( image->Bands ) {
+		case 1:
+		case 2:
+			expected_sig = cmsSigGrayData;
+			break;
+
+		case 3:
+			expected_sig = cmsSigRgbData;
+			break;
+
+		case 4:
+		case 5:
+			expected_sig = cmsSigCmykData;
+			break;
+
+		default:
+			expected_sig = -1;
+			break;
+		}
+		break;
+
 	case VIPS_INTERPRETATION_LCH:
 	case VIPS_INTERPRETATION_CMC:
-	case VIPS_INTERPRETATION_MULTIBAND:
 	case VIPS_INTERPRETATION_HISTOGRAM:
 	case VIPS_INTERPRETATION_MATRIX:
 	case VIPS_INTERPRETATION_FOURIER:
@@ -1226,6 +1250,38 @@ vips_icc_ac2rc( VipsImage *in, VipsImage **out, const char *profile_filename )
 	return( 0 );
 }
 
+/* TRUE if a profile is compatible with an image.
+ */
+gboolean
+vips_icc_is_compatible_profile( VipsImage *image, 
+	void *data, size_t data_length )
+{
+	cmsHPROFILE profile;
+
+	if( !(profile = cmsOpenProfileFromMem( data, data_length )) ) {
+		g_warning( "%s", _( "corrupt profile" ) );
+		return( FALSE ); 
+	}
+
+	if( vips_image_expected_bands( image ) != 
+		vips_icc_profile_needs_bands( profile ) ) {
+		VIPS_FREEF( cmsCloseProfile, profile );
+		g_warning( "%s", 
+			_( "profile incompatible with image" ) );
+		return( FALSE );
+	}
+	if( vips_image_expected_sig( image ) != cmsGetColorSpace( profile ) ) {
+		VIPS_FREEF( cmsCloseProfile, profile );
+		g_warning( "%s", 
+			_( "profile colourspace differs from image" ) );
+		return( FALSE );
+	}
+
+	VIPS_FREEF( cmsCloseProfile, profile );
+
+	return( TRUE );
+}
+
 #else /*!HAVE_LCMS2*/
 
 #include <vips/vips.h>
@@ -1245,6 +1301,13 @@ vips_icc_ac2rc( VipsImage *in, VipsImage **out, const char *profile_filename )
 	return( -1 );
 }
 
+gboolean
+vips_icc_is_compatible_profile( VipsImage *image, 
+	void *data, size_t data_length )
+{
+	return( TRUE ); 
+}
+
 #endif /*HAVE_LCMS*/
 
 /**
