@@ -168,6 +168,8 @@ vips_convolution_operation_init( void )
 	extern int vips_spcor_get_type( void ); 
 	extern int vips_sharpen_get_type( void ); 
 	extern int vips_gaussblur_get_type( void ); 
+	extern int vips_sobel_get_type( void ); 
+	extern int vips_canny_get_type( void ); 
 
 	vips_conv_get_type(); 
 	vips_conva_get_type(); 
@@ -180,4 +182,6 @@ vips_convolution_operation_init( void )
 	vips_spcor_get_type(); 
 	vips_sharpen_get_type(); 
 	vips_gaussblur_get_type(); 
+	vips_canny_get_type(); 
+	vips_sobel_get_type(); 
 }
