@@ -36,6 +36,7 @@
 #include <stdio.h>
 
 #include <vips/vips.h>
+#include <vips/vips7compat.h>
 #include <vips/internal.h>
 
 /* To iterate over supported formats, we build a temp list of subclasses of 
@@ -306,6 +307,8 @@ im__format_init( void )
 #endif /*HAVE_TIFF*/
 	extern GType vips_format_openslide_get_type( void );
 	vips_format_openslide_get_type();
+	extern GType vips_format_nifti_get_type( void );
+	vips_format_nifti_get_type();
 }
 
 /* Can this format open this file?
