@@ -468,6 +468,9 @@ vips_draw_flood_build( VipsObject *object )
 	/* @test defaults to @image.
 	 */
 	if( !vips_object_argument_isset( object, "test" ) )
+		/* Setting input params like this will break caching, but we
+		 * don't cache draw ops anyway.
+		 */
 		g_object_set( object, "test", draw->image, NULL ); 
 
 	if( vips_image_wio_input( drawflood->test ) ||
@@ -667,7 +670,7 @@ vips_draw_floodv( VipsImage *image,
 	VipsArea *area_ink;
 	int result;
 
-	area_ink = (VipsArea *) vips_array_double_new( ink, n );
+	area_ink = VIPS_AREA( vips_array_double_new( ink, n ) );
 	result = vips_call_split( "draw_flood", ap, image, area_ink, x, y );
 	vips_area_unref( area_ink );
 
@@ -675,7 +678,7 @@ vips_draw_floodv( VipsImage *image,
 }
 
 /**
- * vips_draw_flood:
+ * vips_draw_flood: (method)
  * @image: image to draw on
  * @ink: (array length=n): value to draw
  * @n: length of ink array
@@ -727,7 +730,7 @@ vips_draw_flood( VipsImage *image,
 }
 
 /**
- * vips_draw_flood1:
+ * vips_draw_flood1: (method)
  * @image: image to draw on
  * @ink: value to draw
  * @x: centre of circle
