@@ -18,8 +18,12 @@
  * 	- transform cmyk->rgb if there's an embedded profile
  * 16/6/17
  * 	- add page_height
+ * 1/1/18
+ * 	- META_SEQ support moved here
  * 5/3/18
  * 	- block _start if one start fails, see #893
+ * 1/4/18
+ * 	- drop incompatible ICC profiles before save
  */
 
 /*
@@ -720,6 +724,20 @@ vips_foreign_load_temp( VipsForeignLoad *load )
 	const guint64 disc_threshold = vips_get_disc_threshold();
 	const guint64 image_size = VIPS_IMAGE_SIZEOF_IMAGE( load->out );
 
+	/* ->memory used to be called ->disc and default TRUE. If it's been
+	 * forced FALSE, set memory TRUE.
+	 */
+	if( !load->disc )
+		load->memory = TRUE;
+
+	if( load->memory ) {
+#ifdef DEBUG
+		printf( "vips_foreign_load_temp: forced memory temp\n" );
+#endif /*DEBUG*/
+
+		return( vips_image_new_memory() );
+	}
+
 	/* If this is a partial operation, we can open directly.
 	 */
 	if( load->flags & VIPS_FOREIGN_PARTIAL ) {
@@ -742,19 +760,10 @@ vips_foreign_load_temp( VipsForeignLoad *load )
 		return( vips_image_new() );
 	}
 
-	/* ->memory used to be called ->disc and default TRUE. If it's been
-	 * forced FALSE, set memory TRUE.
+	/* We open via disc if the uncompressed image will be larger than 
+	 * vips_get_disc_threshold()
 	 */
-	if( !load->disc )
-		load->memory = TRUE;
-
-	/* We open via disc if:
-	 * - 'memory' is off
-	 * - the uncompressed image will be larger than 
-	 *   vips_get_disc_threshold()
-	 */
-	if( !load->memory && 
-		image_size > disc_threshold ) {
+	if( image_size > disc_threshold ) {
 #ifdef DEBUG
 		printf( "vips_foreign_load_temp: disc temp\n" );
 #endif /*DEBUG*/
@@ -763,12 +772,13 @@ vips_foreign_load_temp( VipsForeignLoad *load )
 	}
 
 #ifdef DEBUG
-	printf( "vips_foreign_load_temp: memory temp\n" );
+	printf( "vips_foreign_load_temp: fallback memory temp\n" );
 #endif /*DEBUG*/
 
 	/* Otherwise, fall back to a memory buffer.
 	 */
 	return( vips_image_new_memory() );
+
 }
 
 /* Check two images for compatibility: their geometries need to match.
@@ -887,6 +897,7 @@ vips_foreign_load_build( VipsObject *object )
 	VipsForeignLoadClass *fclass = VIPS_FOREIGN_LOAD_GET_CLASS( object );
 
 	VipsForeignFlags flags;
+	gboolean sequential;
 
 #ifdef DEBUG
 	printf( "vips_foreign_load_build:\n" );
@@ -906,11 +917,14 @@ vips_foreign_load_build( VipsObject *object )
 
 	g_object_set( load, "flags", flags, NULL );
 
-	/* If the loader can do sequential mode and sequential has been
-	 * requested, we need to block caching.
+	/* Seq access has been requested and the loader supports seq.
 	 */
-	if( (load->flags & VIPS_FOREIGN_SEQUENTIAL) && 
-		load->access != VIPS_ACCESS_RANDOM ) 
+	sequential = (load->flags & VIPS_FOREIGN_SEQUENTIAL) && 
+		load->access != VIPS_ACCESS_RANDOM;
+
+	/* We must block caching of seq loads.
+	 */
+	if( sequential )
 		load->nocache = TRUE;
 
 	if( VIPS_OBJECT_CLASS( vips_foreign_load_parent_class )->
@@ -965,11 +979,11 @@ vips_foreign_load_build( VipsObject *object )
 			return( -1 );
 	}
 
-	/* If random access has been requested, make sure that we don't have a
-	 * SEQ tag left from a sequential loader.
+	/* Tell downstream if we are reading sequentially.
 	 */
-	if( load->access == VIPS_ACCESS_RANDOM ) 
-		(void) vips_image_remove( load->out, VIPS_META_SEQUENTIAL );
+	if( sequential ) 
+		vips_image_set_area( load->out, 
+			VIPS_META_SEQUENTIAL, NULL, NULL ); 
 
 	return( 0 );
 }
@@ -1498,6 +1512,21 @@ vips__foreign_convert_saveable( VipsImage *in, VipsImage **ready,
 		in = out;
 	}
 
+	/* Some format libraries, like libpng, will throw a hard error if the 
+	 * profile is inappropriate for this image type. With profiles inherited
+	 * from a source image, this can happen all the time, so we 
+	 * want to just drop the profile in this case.
+	 */
+	if( vips_image_get_typeof( in, VIPS_META_ICC_NAME ) ) {
+		void *data;
+		size_t length;
+
+		if( !vips_image_get_blob( in, VIPS_META_ICC_NAME, 
+			&data, &length ) &&
+			!vips_icc_is_compatible_profile( in, data, length ) ) 
+			vips_image_remove( in, VIPS_META_ICC_NAME );
+	}
+
 	*ready = in;
 
 	return( 0 );
@@ -1793,6 +1822,8 @@ vips_foreign_operation_init( void )
 	extern GType vips_foreign_load_magick_buffer_get_type( void ); 
 	extern GType vips_foreign_load_magick7_file_get_type( void ); 
 	extern GType vips_foreign_load_magick7_buffer_get_type( void ); 
+	extern GType vips_foreign_save_magick_file_get_type( void );
+	extern GType vips_foreign_save_magick_buffer_get_type( void );
 	extern GType vips_foreign_save_dz_file_get_type( void ); 
 	extern GType vips_foreign_save_dz_buffer_get_type( void ); 
 	extern GType vips_foreign_load_webp_file_get_type( void ); 
@@ -1805,6 +1836,8 @@ vips_foreign_operation_init( void )
 	extern GType vips_foreign_load_svg_get_type( void ); 
 	extern GType vips_foreign_load_svg_file_get_type( void ); 
 	extern GType vips_foreign_load_svg_buffer_get_type( void ); 
+	extern GType vips_foreign_load_nifti_get_type( void ); 
+	extern GType vips_foreign_save_nifti_get_type( void ); 
 	extern GType vips_foreign_load_gif_get_type( void ); 
 	extern GType vips_foreign_load_gif_file_get_type( void ); 
 	extern GType vips_foreign_load_gif_buffer_get_type( void ); 
@@ -1841,6 +1874,12 @@ vips_foreign_operation_init( void )
 	vips_foreign_load_pdf_buffer_get_type(); 
 #endif /*HAVE_POPPLER*/
 
+#ifdef HAVE_PDFIUM
+	vips_foreign_load_pdf_get_type(); 
+	vips_foreign_load_pdf_file_get_type(); 
+	vips_foreign_load_pdf_buffer_get_type(); 
+#endif /*HAVE_PDFIUM*/
+
 #ifdef HAVE_RSVG
 	vips_foreign_load_svg_get_type(); 
 	vips_foreign_load_svg_file_get_type(); 
@@ -1895,15 +1934,22 @@ vips_foreign_operation_init( void )
 	vips_foreign_load_openslide_get_type(); 
 #endif /*HAVE_OPENSLIDE*/
 
-#ifdef HAVE_MAGICK
-	vips_foreign_load_magick_file_get_type(); 
-	vips_foreign_load_magick_buffer_get_type(); 
-#endif /*HAVE_MAGICK*/
+#ifdef ENABLE_MAGICKLOAD
+#ifdef HAVE_MAGICK6
+	vips_foreign_load_magick_file_get_type();
+	vips_foreign_load_magick_buffer_get_type();
+#endif /*HAVE_MAGICK6*/
 
 #ifdef HAVE_MAGICK7
-	vips_foreign_load_magick7_file_get_type(); 
-	vips_foreign_load_magick7_buffer_get_type(); 
+	vips_foreign_load_magick7_file_get_type();
+	vips_foreign_load_magick7_buffer_get_type();
 #endif /*HAVE_MAGICK7*/
+#endif /*ENABLE_MAGICKLOAD*/
+
+#ifdef ENABLE_MAGICKSAVE
+	vips_foreign_save_magick_file_get_type();
+	vips_foreign_save_magick_buffer_get_type();
+#endif /*ENABLE_MAGICKSAVE*/
 
 #ifdef HAVE_CFITSIO
 	vips_foreign_load_fits_get_type(); 
@@ -1914,6 +1960,11 @@ vips_foreign_operation_init( void )
 	vips_foreign_load_openexr_get_type(); 
 #endif /*HAVE_OPENEXR*/
 
+#ifdef HAVE_NIFTI
+	vips_foreign_load_nifti_get_type(); 
+	vips_foreign_save_nifti_get_type(); 
+#endif /*HAVE_NIFTI*/
+
 	vips__foreign_load_operation = 
 		g_quark_from_static_string( "vips-foreign-load-operation" ); 
 }
