@@ -18,6 +18,8 @@
  * 21/11/17
  * 	- add "gif-delay", "gif-loop", "gif-comment" metadata
  * 	- add dispose handling
+ * 13/8/18
+ * 	- init pages to 0 before load
  */
 
 /*
@@ -383,7 +385,7 @@ vips_foreign_load_gif_is_a( const char *filename )
 {
 	unsigned char buf[4];
 
-	if( vips__get_bytes( filename, buf, 4 ) &&
+	if( vips__get_bytes( filename, buf, 4 ) == 4 &&
 		vips_foreign_load_gif_is_a_buffer( buf, 4 ) )
 		return( 1 );
 
@@ -779,6 +781,13 @@ vips_foreign_load_gif_new_page( VipsForeignLoadGif *gif )
 		return( NULL );
 	}
 
+	/* Some GIFs may not clear the background, so we must start
+	 * transparent.
+	 */
+	memset( VIPS_IMAGE_ADDR( out, 0, 0 ), 
+		0, 
+		VIPS_IMAGE_SIZEOF_IMAGE( out ) );
+
 	return( out );
 }
 
