@@ -94,8 +94,12 @@
  * 	- revert previous warning change: libvips reports serious corruption, 
  * 	  like a truncated file, as a warning and we need to be able to catch
  * 	  that
+ * 9/4/18
+ * 	- set interlaced=1 for interlaced images
  * 10/4/18
  * 	- strict round down on shrink-on-load
+ * 16/8/18
+ * 	- shut down the input file as soon as we can [kleisauke]
  */
 
 /*
@@ -186,15 +190,30 @@ typedef struct _ReadJpeg {
 	int output_height;
 } ReadJpeg;
 
+/* This can be called many times. It's called directly at the end of image
+ * read.
+ */
+static void
+readjpeg_close_input( ReadJpeg *jpeg )
+{
+	VIPS_FREEF( fclose, jpeg->eman.fp );
+
+	/* Don't call jpeg_finish_decompress(). It just checks the tail of the
+	 * file and who cares about that. All mem is freed in
+	 * jpeg_destroy_decompress().
+	 */
+
+	/* I don't think this can fail. It's harmless to call many times. 
+	 */
+	jpeg_destroy_decompress( &jpeg->cinfo );
+
+}
+
 /* This can be called many times.
  */
 static int
 readjpeg_free( ReadJpeg *jpeg )
 {
-	int result;
-
-	result = 0;
-
 	if( jpeg->eman.pub.num_warnings != 0 ) {
 		g_warning( _( "read gave %ld warnings" ), 
 			jpeg->eman.pub.num_warnings );
@@ -205,24 +224,15 @@ readjpeg_free( ReadJpeg *jpeg )
 		jpeg->eman.pub.num_warnings = 0;
 	}
 
-	/* Don't call jpeg_finish_decompress(). It just checks the tail of the
-	 * file and who cares about that. All mem is freed in
-	 * jpeg_destroy_decompress().
-	 */
+	readjpeg_close_input( jpeg );
 
-	VIPS_FREEF( fclose, jpeg->eman.fp );
 	VIPS_FREE( jpeg->filename );
-	jpeg->eman.fp = NULL;
 
-	/* I don't think this can fail. It's harmless to call many times. 
-	 */
-	jpeg_destroy_decompress( &jpeg->cinfo );
-
-	return( result );
+	return( 0 );
 }
 
 static void
-readjpeg_close( VipsObject *object, ReadJpeg *jpeg )
+readjpeg_close_cb( VipsObject *object, ReadJpeg *jpeg )
 {
 	(void) readjpeg_free( jpeg );
 }
@@ -259,7 +269,7 @@ readjpeg_new( VipsImage *out, int shrink, gboolean fail, gboolean autorotate )
         jpeg_create_decompress( &jpeg->cinfo );
 
 	g_signal_connect( out, "close", 
-		G_CALLBACK( readjpeg_close ), jpeg ); 
+		G_CALLBACK( readjpeg_close_cb ), jpeg ); 
 
 	return( jpeg );
 }
@@ -277,6 +287,25 @@ readjpeg_file( ReadJpeg *jpeg, const char *filename )
 	return( 0 );
 }
 
+static const char *
+find_chroma_subsample( struct jpeg_decompress_struct *cinfo )
+{
+	gboolean has_subsample;
+
+	/* libjpeg only uses 4:4:4 and 4:2:0, confusingly. 
+	 *
+	 * http://poynton.ca/PDFs/Chroma_subsampling_notation.pdf
+	 */
+	has_subsample = cinfo->max_h_samp_factor > 1 ||
+		cinfo->max_v_samp_factor > 1;
+	if( cinfo->num_components > 3 )
+		/* A cmyk image.
+		 */
+		return( has_subsample ? "4:2:0:4" : "4:4:4:4" );
+	else
+		return( has_subsample ? "4:2:0" : "4:4:4" );
+}
+
 static int
 attach_blob( VipsImage *im, const char *field, void *data, int data_length )
 {
@@ -432,6 +461,14 @@ read_jpeg_header( ReadJpeg *jpeg, VipsImage *out )
 	(void) vips_image_set_int( out, "jpeg-multiscan", 
 		jpeg_has_multiple_scans( cinfo ) );
 
+	/* 8.7 adds this for PNG as well, so we have a new format-neutral name.
+	 */
+	if( jpeg_has_multiple_scans( cinfo ) )
+		vips_image_set_int( out, "interlaced", 1 ); 
+
+	(void) vips_image_set_string( out, "jpeg-chroma-subsample", 
+		find_chroma_subsample( cinfo ) );
+
 	/* Look for EXIF and ICC profile.
 	 */
 	for( p = cinfo->marker_list; p; p = p->next ) {
@@ -541,10 +578,6 @@ read_jpeg_header( ReadJpeg *jpeg, VipsImage *out )
 			(VipsCallbackFn) vips_free, data, data_length );
 	}
 
-	/* Tell downstream we are reading sequentially.
-	 */
-	vips_image_set_area( out, VIPS_META_SEQUENTIAL, NULL, NULL ); 
-
 	return( 0 );
 }
 
@@ -639,11 +672,10 @@ read_jpeg_generate( VipsRegion *or,
 		jpeg->y_pos += 1; 
 	}
 
-	/* Progressive images can have a lot of memory in the decompress
-	 * object, destroy as soon as we can. Safe to call many times. 
+	/* Shut down the input file as soon as we can. 
 	 */
 	if( jpeg->y_pos >= or->im->Ysize ) 
-		jpeg_destroy_decompress( &jpeg->cinfo );
+		readjpeg_close_input( jpeg );
 
 	VIPS_GATE_STOP( "read_jpeg_generate: work" );
 
@@ -1000,7 +1032,7 @@ vips__isjpeg( const char *filename )
 {
 	unsigned char buf[2];
 
-	if( vips__get_bytes( filename, buf, 2 ) &&
+	if( vips__get_bytes( filename, buf, 2 ) == 2 &&
 		vips__isjpeg_buffer( buf, 2 ) )
 		return( 1 );
 
