@@ -58,7 +58,9 @@
 #include <vips/buf.h>
 #include <vips/internal.h>
 
-#ifdef HAVE_MAGICK
+#ifdef ENABLE_MAGICKLOAD
+
+#ifdef HAVE_MAGICK6
 
 #include "pforeign.h"
 
@@ -120,28 +122,28 @@ vips_foreign_load_magick_class_init( VipsForeignLoadMagickClass *class )
 		vips_foreign_load_magick_get_flags_filename;
 	load_class->get_flags = vips_foreign_load_magick_get_flags;
 
-	VIPS_ARG_BOOL( class, "all_frames", 3, 
+	VIPS_ARG_BOOL( class, "all_frames", 4, 
 		_( "all_frames" ), 
 		_( "Read all frames from an image" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
 		G_STRUCT_OFFSET( VipsForeignLoadMagick, all_frames ),
 		FALSE );
 
-	VIPS_ARG_STRING( class, "density", 4,
+	VIPS_ARG_STRING( class, "density", 5,
 		_( "Density" ),
 		_( "Canvas resolution for rendering vector formats like SVG" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadMagick, density ),
 		NULL );
 
-	VIPS_ARG_INT( class, "page", 5,
+	VIPS_ARG_INT( class, "page", 6,
 		_( "Page" ),
 		_( "Load this page from the file" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadMagick, page ),
 		0, 100000, 0 );
 
-	VIPS_ARG_INT( class, "n", 6,
+	VIPS_ARG_INT( class, "n", 7,
 		_( "n" ),
 		_( "Load this many pages" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
@@ -200,7 +202,8 @@ vips_foreign_load_magick_file_header( VipsForeignLoad *load )
 		magick->n = -1;
 
 	if( vips__magick_read( magick_file->filename, 
-		load->out, magick->density, magick->page, magick->n ) )
+		load->out, magick->density, 
+		magick->page, magick->n ) )
 		return( -1 );
 
 	VIPS_SETSTR( load->out->filename, magick_file->filename );
@@ -286,7 +289,8 @@ vips_foreign_load_magick_buffer_header( VipsForeignLoad *load )
 
 	if( vips__magick_read_buffer( 
 		magick_buffer->buf->data, magick_buffer->buf->length, 
-		load->out, magick->density, magick->page, magick->n ) )
+		load->out, magick->density, magick->page, 
+		magick->n ) )
 		return( -1 );
 
 	return( 0 );
@@ -324,7 +328,9 @@ vips_foreign_load_magick_buffer_init( VipsForeignLoadMagickBuffer *buffer )
 {
 }
 
-#endif /*HAVE_MAGICK*/
+#endif /*HAVE_MAGICK6*/
+
+#endif /*ENABLE_MAGICKLOAD*/
 
 /**
  * vips_magickload:
@@ -348,6 +354,9 @@ vips_foreign_load_magick_buffer_init( VipsForeignLoadMagickBuffer *buffer )
  * The reader should also work with most versions of GraphicsMagick. See the
  * "--with-magickpackage" configure option.
  *
+ * The file format is usually guessed from the filename suffix, or sniffed
+ * from the file contents.
+ *
  * Normally it will only load the first image in a many-image sequence (such
  * as a GIF or a PDF). Use @page and @n to set the start page and number of
  * pages to load. Set @n to -1 to load all pages from @page onwards.
