@@ -14,6 +14,10 @@
  * 	- redo as a set of fns ready for wrapping in a new-style class
  * 17/9/16
  * 	- tag output as scRGB
+ * 16/8/18
+ * 	- shut down the input file as soon as we can [kleisauke]
+ * 19/8/18
+ * 	- scale alpha up to 0 - 255 to match the rest of libvips
  */
 
 /*
@@ -107,7 +111,7 @@ vips__openexr_isexr( const char *filename )
 {
 	unsigned char buf[4];
 
-	if( vips__get_bytes( filename, buf, 4 ) )
+	if( vips__get_bytes( filename, buf, 4 ) == 4 )
 		if( buf[0] == 0x76 && buf[1] == 0x2f &&
 			buf[2] == 0x31 && buf[3] == 0x01 )
 			return( TRUE );
@@ -121,13 +125,19 @@ get_imf_error( void )
 	vips_error( "exr2vips", _( "EXR error: %s" ), ImfErrorMessage() );
 }
 
+static void
+read_close( Read *read )
+{
+	VIPS_FREEF( ImfCloseTiledInputFile, read->tiles );
+	VIPS_FREEF( ImfCloseInputFile, read->lines );
+}
+
 static void
 read_destroy( VipsImage *out, Read *read )
 {
 	VIPS_FREE( read->filename );
 
-	VIPS_FREEF( ImfCloseTiledInputFile, read->tiles );
-	VIPS_FREEF( ImfCloseInputFile, read->lines );
+	read_close( read ); 
 
 	vips_free( read );
 }
@@ -229,6 +239,7 @@ vips__openexr_read_header( const char *filename, VipsImage *out )
 	if( !(read = read_new( filename, out )) )
 		return( -1 );
 	read_header( read, out );
+	read_close( read );
 
 	return( 0 );
 }
@@ -324,11 +335,24 @@ vips__openexr_generate( VipsRegion *out,
 				float *q = (float *) VIPS_REGION_ADDR( out,
 					hit.left, hit.top + z );
 
+				int i;
+
 				ImfHalfToFloatArray( 4 * hit.width, 
 					(ImfHalf *) p, q );
+
+				/* oexr uses 0 - 1 for alpha, but vips is 
+				 * always 0 - 255, even for scrgb images.
+				 */
+				for( i = 0; i < hit.width; i++ ) 
+					q[4 * i + 3] *= 255;
 			}
 		}
 
+	/* We can't shut down the input file early for tile read, even if we
+	 * know load is in sequential mode, since we are not inside a
+	 * vips_sequential() and requests are not guaranteed to be in order.
+	 */
+
 	return( 0 );
 }
 
@@ -389,6 +413,8 @@ vips__openexr_read( const char *filename, VipsImage *out )
 		read_header( read, out );
 
 		for( y = 0; y < height; y++ ) {
+			int i;
+
 			if( !ImfInputSetFrameBuffer( read->lines,
 				imf_buffer - left - (top + y) * width,
 				1, width ) ) {
@@ -404,10 +430,18 @@ vips__openexr_read( const char *filename, VipsImage *out )
 			ImfHalfToFloatArray( 4 * width, 
 				(ImfHalf *) imf_buffer, vips_buffer );
 
+			/* oexr uses 0 - 1 for alpha, but vips is always 0 -
+			 * 255, even for scrgb images.
+			 */
+			for( i = 0; i < width; i++ ) 
+				vips_buffer[4 * i + 3] *= 255;
+
 			if( vips_image_write_line( out, y, 
 				(VipsPel *) vips_buffer ) )
 				return( -1 );
 		}
+
+		read_close( read ); 
 	}
 
 	return( 0 );
