@@ -177,8 +177,12 @@
  * 	- remove missing res warning
  * 19/5/17
  * 	- page > 0 could break edge tiles or strips
+ * 26/4/18
+ * 	- add n-pages metadata item
  * 21/7/18
  * 	- check for non-byte-multiple bits_per_sample [HongxuChen]
+ * 16/8/18
+ * 	- shut down the input file as soon as we can [kleisauke]
  */
 
 /*
@@ -285,6 +289,10 @@ typedef struct _Rtiff {
 	 */
 	TIFF *tiff;
 
+	/* Number of pages (directories) in image.
+	 */
+	int n_pages;
+
 	/* The current page we have set.
 	 */
 	int current_page;
@@ -449,6 +457,64 @@ get_orientation( TIFF *tiff )
 	return( orientation );
 }
 
+/* Can be called many times.
+ */
+static void
+rtiff_free( Rtiff *rtiff )
+{
+	VIPS_FREEF( TIFFClose, rtiff->tiff );
+}
+
+static void
+rtiff_close_cb( VipsObject *object, Rtiff *rtiff )
+{
+	rtiff_free( rtiff ); 
+}
+
+static Rtiff *
+rtiff_new( VipsImage *out, int page, int n, gboolean autorotate )
+{
+	Rtiff *rtiff;
+
+	if( !(rtiff = VIPS_NEW( out, Rtiff )) )
+		return( NULL );
+
+	rtiff->filename = NULL;
+	rtiff->out = out;
+	rtiff->page = page;
+	rtiff->n = n;
+	rtiff->autorotate = autorotate;
+	rtiff->tiff = NULL;
+	rtiff->n_pages = 0;
+	rtiff->current_page = -1;
+	rtiff->sfn = NULL;
+	rtiff->client = NULL;
+	rtiff->memcpy = FALSE;
+	rtiff->plane_buf = NULL;
+	rtiff->contig_buf = NULL;
+
+	g_signal_connect( out, "close", 
+		G_CALLBACK( rtiff_close_cb ), rtiff ); 
+
+	if( rtiff->page < 0 || rtiff->page > 1000000 ) {
+		vips_error( "tiff2vips", _( "bad page number %d" ),
+			rtiff->page );
+		return( NULL );
+	}
+
+	/* We allow n == -1, meaning all pages. It gets swapped for a real n
+	 * value when we open the TIFF.
+	 */
+	if( rtiff->n != -1 &&
+		(rtiff->n < 1 || rtiff->n > 1000000) ) {
+		vips_error( "tiff2vips", _( "bad number of pages %d" ),
+			rtiff->n );
+		return( NULL );
+	}
+
+	return( rtiff );
+}
+
 static int
 rtiff_strip_read( Rtiff *rtiff, int strip, tdata_t buf )
 {
@@ -540,7 +606,7 @@ rtiff_check_min_samples( Rtiff *rtiff, int samples_per_pixel )
 	return( 0 );
 }
 
-/* Only allow samples which are whole bytes in size.
+/* Only allow samples which are whole bytes in size.  
  */
 static int
 rtiff_non_fractional( Rtiff *rtiff )
@@ -1252,6 +1318,8 @@ rtiff_set_header( Rtiff *rtiff, VipsImage *out )
 		vips_image_set_int( out, 
 			VIPS_META_PAGE_HEIGHT, rtiff->header.height );
 
+	vips_image_set_int( out, VIPS_META_N_PAGES, rtiff->n_pages );
+
 	/* Even though we could end up serving tiled data, always hint
 	 * THINSTRIP, since we're quite happy doing that too, and it could need
 	 * a lot less memory.
@@ -1367,11 +1435,6 @@ rtiff_set_header( Rtiff *rtiff, VipsImage *out )
 	vips_image_set_int( out, 
 		VIPS_META_ORIENTATION, rtiff->header.orientation );
 
-	/* Tell downstream if we are reading sequentially.
-	 */
-	if( !rtiff->header.tiled ) 
-		vips_image_set_area( out, VIPS_META_SEQUENTIAL, NULL, NULL ); 
-
 	return( 0 );
 }
 
@@ -1567,6 +1630,11 @@ rtiff_fill_region( VipsRegion *out,
 
 	VIPS_GATE_STOP( "rtiff_fill_region: work" ); 
 
+	/* We can't shut down the input file early for tile read, even if we
+	 * know load is in sequential mode, since we are not inside a
+	 * vips_sequential() and requests are not guaranteed to be in order.
+	 */
+
 	return( 0 );
 }
 
@@ -1862,7 +1930,7 @@ rtiff_stripwise_generate( VipsRegion *or,
 
 			/* Do any repacking to generate pixels in vips layout.
 			 */
-			p = rtiff->contig_buf + 
+			p = (VipsPel *) rtiff->contig_buf + 
 				(hit.top - strip.top) * scanline_size;
 			q = VIPS_REGION_ADDR( or, 0, r->top + y );
 			for( z = 0; z < hit.height; z++ ) { 
@@ -1877,6 +1945,11 @@ rtiff_stripwise_generate( VipsRegion *or,
 		y += hit.height;
 	}
 
+	/* Shut down the input file as soon as we can. 
+	 */
+	if( r->top + y >= or->im->Ysize ) 
+		rtiff_free( rtiff );
+
 	VIPS_GATE_STOP( "rtiff_stripwise_generate: work" ); 
 
 	return( 0 );
@@ -1983,63 +2056,6 @@ rtiff_read_stripwise( Rtiff *rtiff, VipsImage *out )
 	return( 0 );
 }
 
-/* Can be called many times.
- */
-static void
-rtiff_free( Rtiff *rtiff )
-{
-	VIPS_FREEF( TIFFClose, rtiff->tiff );
-}
-
-static void
-rtiff_close( VipsObject *object, Rtiff *rtiff )
-{
-	rtiff_free( rtiff ); 
-}
-
-static Rtiff *
-rtiff_new( VipsImage *out, int page, int n, gboolean autorotate )
-{
-	Rtiff *rtiff;
-
-	if( !(rtiff = VIPS_NEW( out, Rtiff )) )
-		return( NULL );
-
-	rtiff->filename = NULL;
-	rtiff->out = out;
-	rtiff->page = page;
-	rtiff->n = n;
-	rtiff->autorotate = autorotate;
-	rtiff->tiff = NULL;
-	rtiff->current_page = -1;
-	rtiff->sfn = NULL;
-	rtiff->client = NULL;
-	rtiff->memcpy = FALSE;
-	rtiff->plane_buf = NULL;
-	rtiff->contig_buf = NULL;
-
-	g_signal_connect( out, "close", 
-		G_CALLBACK( rtiff_close ), rtiff ); 
-
-	if( rtiff->page < 0 || rtiff->page > 1000000 ) {
-		vips_error( "tiff2vips", _( "bad page number %d" ),
-			rtiff->page );
-		return( NULL );
-	}
-
-	/* We allow n == -1, meaning all pages. It gets swapped for a real n
-	 * value when we open the TIFF.
-	 */
-	if( rtiff->n != -1 &&
-		(rtiff->n < 1 || rtiff->n > 1000000) ) {
-		vips_error( "tiff2vips", _( "bad number of pages %d" ),
-			rtiff->n );
-		return( NULL );
-	}
-
-	return( rtiff );
-}
-
 /* Load from a tiff dir into one of our tiff header structs.
  */
 static int
@@ -2194,8 +2210,9 @@ rtiff_header_read_all( Rtiff *rtiff )
 
 	/* -1 means "to the end".
 	 */
+	rtiff->n_pages = rtiff_n_pages( rtiff );
 	if( rtiff->n == -1 )
-		rtiff->n = rtiff_n_pages( rtiff ) - rtiff->page;
+		rtiff->n = rtiff->n_pages - rtiff->page;
 
 	/* If we're to read many pages, verify that they are all identical. 
 	 */
@@ -2336,7 +2353,8 @@ vips__tiff_read_header( const char *filename, VipsImage *out,
 
 	vips__tiff_init();
 
-	if( !(rtiff = rtiff_new_filename( filename, out, page, n, autorotate )) )
+	if( !(rtiff = 
+		rtiff_new_filename( filename, out, page, n, autorotate )) )
 		return( -1 );
 
 	if( rtiff_set_header( rtiff, out ) )
@@ -2389,7 +2407,7 @@ vips__istiff( const char *filename )
 {
 	unsigned char buf[4];
 
-	if( vips__get_bytes( filename, buf, 4 ) &&
+	if( vips__get_bytes( filename, buf, 4 ) == 4 &&
 		vips__istiff_buffer( buf, 4 ) )
 		return( TRUE );
 
