@@ -85,6 +85,7 @@ typedef struct _VipsForeignSaveTiff {
 	gboolean bigtiff;
 	gboolean rgbjpeg;
 	gboolean properties;
+	VipsRegionShrink region_shrink;
 } VipsForeignSaveTiff;
 
 typedef VipsForeignSaveClass VipsForeignSaveTiffClass;
@@ -293,6 +294,13 @@ vips_foreign_save_tiff_class_init( VipsForeignSaveTiffClass *class )
 		G_STRUCT_OFFSET( VipsForeignSaveTiff, properties ),
 		FALSE );
 
+	VIPS_ARG_ENUM( class, "region_shrink", 22,
+		_( "Region shrink" ),
+		_( "Method to shrink regions" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSaveTiff, region_shrink ),
+		VIPS_TYPE_REGION_SHRINK, VIPS_REGION_SHRINK_MEAN ); 
+
 }
 
 static void
@@ -306,6 +314,7 @@ vips_foreign_save_tiff_init( VipsForeignSaveTiff *tiff )
 	tiff->resunit = VIPS_FOREIGN_TIFF_RESUNIT_CM;
 	tiff->xres = 1.0;
 	tiff->yres = 1.0;
+	tiff->region_shrink = VIPS_REGION_SHRINK_MEAN;
 }
 
 typedef struct _VipsForeignSaveTiffFile {
@@ -341,7 +350,8 @@ vips_foreign_save_tiff_file_build( VipsObject *object )
 		tiff->bigtiff,
 		tiff->rgbjpeg,
 		tiff->properties,
-		save->strip ) )
+		save->strip,
+		tiff->region_shrink ) )
 		return( -1 );
 
 	return( 0 );
@@ -409,7 +419,8 @@ vips_foreign_save_tiff_buffer_build( VipsObject *object )
 		tiff->bigtiff,
 		tiff->rgbjpeg,
 		tiff->properties,
-		save->strip ) )
+		save->strip,
+		tiff->region_shrink ) )
 		return( -1 );
 
 	/* vips__tiff_write_buf() makes a buffer that needs g_free(), not
@@ -475,6 +486,7 @@ vips_foreign_save_tiff_buffer_init( VipsForeignSaveTiffBuffer *buffer )
  * * @properties: set %TRUE to write an IMAGEDESCRIPTION tag
  * * @strip: set %TRUE to block metadata save
  * * @page_height: %gint for page height for multi-page save
+ * * @shrink_region: #VipsRegionShrink How to shrink each 2x2 region.
  *
  * Write a VIPS image to a file as TIFF.
  *
@@ -509,7 +521,9 @@ vips_foreign_save_tiff_buffer_init( VipsForeignSaveTiffBuffer *buffer )
  * is 128 by 128.
  *
  * Set @pyramid to write the image as a set of images, one per page, of
- * decreasing size. 
+ * decreasing size. Use @shrink_region to set how images will be shrunk: by
+ * default each 2x2 block is just averaged, but you can set MODE or MEDIAN as
+ * well.
  *
  * Set @squash to make 8-bit uchar images write as 1-bit TIFFs. Values >128
  * are written as white, values <=128 as black. Normally vips will write
@@ -586,6 +600,7 @@ vips_tiffsave( VipsImage *in, const char *filename, ... )
  * * @properties: set %TRUE to write an IMAGEDESCRIPTION tag
  * * @strip: set %TRUE to block metadata save
  * * @page_height: %gint for page height for multi-page save
+ * * @shrink_region: #VipsRegionShrink How to shrink each 2x2 region.
  *
  * As vips_tiffsave(), but save to a memory buffer. 
  *
