@@ -65,6 +65,10 @@
  * 	- better behaviour for truncated png files, thanks Yury
  * 26/4/17
  * 	- better @fail handling with truncated PNGs
+ * 9/4/18
+ * 	- set interlaced=1 for interlaced images
+ * 20/6/18 [felixbuenemann]
+ * 	- support png8 palette write with palette, colours, Q, dither
  */
 
 /*
@@ -397,19 +401,16 @@ png2vips_header( Read *read, VipsImage *out )
 	/* Uninterlaced images will be read in seq mode. Interlaced images are
 	 * read via a huge memory buffer.
 	 */
-	if( interlace_type == PNG_INTERLACE_NONE ) {
-		vips_image_set_area( out, VIPS_META_SEQUENTIAL, NULL, NULL ); 
-
+	if( interlace_type == PNG_INTERLACE_NONE ) 
 		/* Sequential mode needs thinstrip to work with things like
 		 * vips_shrink().
 		 */
 		vips_image_pipelinev( out, VIPS_DEMAND_STYLE_THINSTRIP, NULL );
-	}
 	else 
 		vips_image_pipelinev( out, VIPS_DEMAND_STYLE_ANY, NULL );
 
 	/* Fetch the ICC profile. @name is useless, something like "icc" or
-	 * "ICC Profile" etc.  Ignore it.
+	 * "ICC Profile" etc. Ignore it.
 	 *
 	 * @profile was png_charpp in libpngs < 1.5, png_bytepp is the
 	 * modern one. Ignore the warning, if any.
@@ -441,6 +442,11 @@ png2vips_header( Read *read, VipsImage *out )
 		return( -1 );
 	}
 
+	/* Let our caller know. These are very expensive to decode.
+	 */
+	if( interlace_type != PNG_INTERLACE_NONE ) 
+		vips_image_set_int( out, "interlaced", 1 ); 
+
 	return( 0 );
 }
 
@@ -680,7 +686,7 @@ vips__png_ispng( const char *filename )
 {
 	unsigned char buf[8];
 
-	return( vips__get_bytes( filename, buf, 8 ) &&
+	return( vips__get_bytes( filename, buf, 8 ) == 8 &&
 		vips__png_ispng_buffer( buf, 8 ) ); 
 }
 
@@ -696,7 +702,7 @@ vips_png_read_buffer( png_structp pPng, png_bytep data, png_size_t length )
 	if( read->read_pos + length > read->length )
 		png_error( pPng, "not enough data in buffer" );
 
-	memcpy( data, read->buffer + read->read_pos, length );
+	memcpy( data, (VipsPel *) read->buffer + read->read_pos, length );
 	read->read_pos += length;
 }
 
@@ -878,7 +884,8 @@ write_png_block( VipsRegion *region, VipsRect *area, void *a )
 static int
 write_vips( Write *write, 
 	int compress, int interlace, const char *profile,
-	VipsForeignPngFilter filter, gboolean strip )
+	VipsForeignPngFilter filter, gboolean strip,
+	gboolean palette, int colours, int Q, double dither )
 {
 	VipsImage *in = write->in;
 
@@ -938,6 +945,21 @@ write_vips( Write *write,
 		return( -1 );
 	}
 
+#ifdef HAVE_IMAGEQUANT
+	/* Enable image quantisation to paletted 8bpp PNG if colours is set.
+	 */
+	if( palette ) {
+		g_assert( colours >= 2 && 
+			colours <= 256 );
+		bit_depth = 8;
+		color_type = PNG_COLOR_TYPE_PALETTE;
+	}
+#else
+	if( palette )
+		g_warning( "%s",
+			_( "ignoring palette (no quantisation support)" ) );
+#endif /*HAVE_IMAGEQUANT*/
+
 	interlace_type = interlace ? PNG_INTERLACE_ADAM7 : PNG_INTERLACE_NONE;
 
 	png_set_IHDR( write->pPng, write->pInfo, 
@@ -990,12 +1012,78 @@ write_vips( Write *write,
 			PNG_COMPRESSION_TYPE_BASE, data, length );
 	}
 
-	png_write_info( write->pPng, write->pInfo ); 
+#ifdef HAVE_IMAGEQUANT
+	if( palette ) {
+		VipsImage *im_index;
+		VipsImage *im_palette;
+		int palette_count;
+		png_color *png_palette;
+		png_byte *png_trans;
+		int trans_count;
+
+		if( vips__quantise_image( in, &im_index, &im_palette, 
+			colours, Q, dither ) ) 
+			return( -1 );
+
+		palette_count = im_palette->Xsize;
+
+		g_assert( palette_count <= PNG_MAX_PALETTE_LENGTH );
+
+		png_palette = (png_color *) png_malloc( write->pPng,
+			palette_count * sizeof( png_color ) );
+		png_trans = (png_byte *) png_malloc( write->pPng,
+			palette_count * sizeof( png_byte ) );
+		trans_count = 0;
+		for( i = 0; i < palette_count; i++ ) {
+			VipsPel *p = (VipsPel *) 
+				VIPS_IMAGE_ADDR( im_palette, i, 0 );
+			png_color *col = &png_palette[i];
+
+			col->red = p[0];
+			col->green = p[1];
+			col->blue = p[2];
+			png_trans[i] = p[3];
+			if( p[3] != 255 )
+				trans_count = i + 1;
+#ifdef DEBUG
+			printf( "write_vips: palette[%d] %d %d %d %d\n",
+				i + 1, p[0], p[1], p[2], p[3] );
+#endif /*DEBUG*/
+		}
+
+#ifdef DEBUG
+		printf( "write_vips: attaching %d color palette\n",
+			palette_count );
+#endif /*DEBUG*/
+		png_set_PLTE( write->pPng, write->pInfo, png_palette,
+			palette_count );
+		if( trans_count ) {
+#ifdef DEBUG
+			printf( "write_vips: attaching %d alpha values\n",
+				trans_count );
+#endif /*DEBUG*/
+			png_set_tRNS( write->pPng, write->pInfo, png_trans,
+				trans_count, NULL );
+		}
+
+		png_free( write->pPng, (void *) png_palette );
+		png_free( write->pPng, (void *) png_trans );
+
+		VIPS_UNREF( im_palette );
+
+		VIPS_UNREF( write->memory );
+		write->memory = im_index;
+		in = write->memory;
+	}
+#endif /*HAVE_IMAGEQUANT*/
+
+	png_write_info( write->pPng, write->pInfo );
 
 	/* If we're an intel byte order CPU and this is a 16bit image, we need
 	 * to swap bytes.
 	 */
-	if( bit_depth > 8 && !vips_amiMSBfirst() ) 
+	if( bit_depth > 8 && 
+		!vips_amiMSBfirst() ) 
 		png_set_swap( write->pPng ); 
 
 	if( interlace )	
@@ -1022,7 +1110,8 @@ write_vips( Write *write,
 int
 vips__png_write( VipsImage *in, const char *filename, 
 	int compress, int interlace, const char *profile,
-	VipsForeignPngFilter filter, gboolean strip )
+	VipsForeignPngFilter filter, gboolean strip,
+	gboolean palette, int colours, int Q, double dither )
 {
 	Write *write;
 
@@ -1042,7 +1131,8 @@ vips__png_write( VipsImage *in, const char *filename,
 	/* Convert it!
 	 */
 	if( write_vips( write, 
-		compress, interlace, profile, filter, strip ) ) {
+		compress, interlace, profile, filter, strip, palette,
+		colours, Q, dither ) ) {
 		vips_error( "vips2png", 
 			_( "unable to write \"%s\"" ), filename );
 
@@ -1069,7 +1159,8 @@ user_write_data( png_structp png_ptr, png_bytep data, png_size_t length )
 int
 vips__png_write_buf( VipsImage *in, 
 	void **obuf, size_t *olen, int compression, int interlace,
-	const char *profile, VipsForeignPngFilter filter, gboolean strip )
+	const char *profile, VipsForeignPngFilter filter, gboolean strip,
+	gboolean palette, int colours, int Q, double dither )
 {
 	Write *write;
 
@@ -1081,10 +1172,11 @@ vips__png_write_buf( VipsImage *in,
 	/* Convert it!
 	 */
 	if( write_vips( write, 
-		compression, interlace, profile, filter, strip ) ) {
+		compression, interlace, profile, filter, strip, palette,
+		colours, Q, dither ) ) {
 		vips_error( "vips2png", 
 			"%s", _( "unable to write to buffer" ) );
-	      
+
 		return( -1 );
 	}
 
