@@ -119,7 +119,7 @@ vips__iswebp( const char *filename )
 	 */
 	unsigned char header[12];
 
-	if( vips__get_bytes( filename, header, 12 ) &&
+	if( vips__get_bytes( filename, header, 12 ) == 12 &&
 		vips__iswebp_buffer( header, 12 ) )
 		return( 1 );
 
@@ -244,7 +244,7 @@ read_header( Read *read, VipsImage *out )
 	bitstream.bytes = read->data;
 	bitstream.size = read->length;
 	if( !(mux = WebPMuxCreate( &bitstream, 0 )) ) {
-		vips_warn( "webp", "%s", _( "unable to read image metadata" ) ); 
+		g_warning( "%s", _( "unable to read image metadata" ) ); 
 		return( 0 ); 
 	}
 
