@@ -495,6 +495,10 @@ int vips_magickload( const char *filename, VipsImage **out, ... )
 	__attribute__((sentinel));
 int vips_magickload_buffer( void *buf, size_t len, VipsImage **out, ... )
 	__attribute__((sentinel));
+int vips_magicksave( VipsImage *in, const char *filename, ... )
+	__attribute__((sentinel));
+int vips_magicksave_buffer( VipsImage *in, void **buf, size_t *len, ... )
+	__attribute__((sentinel));
 
 /**
  * VipsForeignPngFilter:
@@ -514,7 +518,7 @@ typedef enum /*< flags >*/ {
 	VIPS_FOREIGN_PNG_FILTER_UP = 0x20,
 	VIPS_FOREIGN_PNG_FILTER_AVG = 0x40,
 	VIPS_FOREIGN_PNG_FILTER_PAETH = 0x80,
-	VIPS_FOREIGN_PNG_FILTER_ALL = 0xEA
+	VIPS_FOREIGN_PNG_FILTER_ALL = 0xF8
 } VipsForeignPngFilter;
 
 int vips_pngload( const char *filename, VipsImage **out, ... )
@@ -556,6 +560,11 @@ int vips_gifload( const char *filename, VipsImage **out, ... )
 int vips_gifload_buffer( void *buf, size_t len, VipsImage **out, ... )
 	__attribute__((sentinel));
 
+int vips_niftiload( const char *filename, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_niftisave( VipsImage *in, const char *filename, ... )
+	__attribute__((sentinel));
+
 /**
  * VipsForeignDzLayout:
  * @VIPS_FOREIGN_DZ_LAYOUT_DZ: use DeepZoom directory layout
