@@ -100,8 +100,6 @@ VIPS_ARGUMENT_OPTIONAL_OUTPUT   Eg. the x pos of the image minimum
 	(VIPS_ARGUMENT_OUTPUT | \
 	 VIPS_ARGUMENT_CONSTRUCT)
 
-extern int _vips__argument_id;
-
 #define VIPS_ARG_IMAGE( CLASS, NAME, PRIORITY, LONG, DESC, FLAGS, OFFSET ) { \
 	GParamSpec *pspec; \
 	\
@@ -109,7 +107,7 @@ extern int _vips__argument_id;
 		VIPS_TYPE_IMAGE, \
 		(GParamFlags) (G_PARAM_READWRITE) ); \
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -121,7 +119,7 @@ extern int _vips__argument_id;
 		VIPS_TYPE_INTERPOLATE, \
 		(GParamFlags) (G_PARAM_READWRITE) ); \
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -134,7 +132,7 @@ extern int _vips__argument_id;
 		(VALUE), \
 		(GParamFlags) (G_PARAM_READWRITE) ); \
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -147,7 +145,7 @@ extern int _vips__argument_id;
 		(MIN), (MAX), (VALUE), \
 		(GParamFlags) (G_PARAM_READWRITE) );\
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -160,7 +158,7 @@ extern int _vips__argument_id;
 		(TYPE), \
 		(GParamFlags) (G_PARAM_READWRITE) );\
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -173,7 +171,7 @@ extern int _vips__argument_id;
 		(MIN), (MAX), (VALUE), \
 		(GParamFlags) (G_PARAM_READWRITE) );\
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -186,7 +184,7 @@ extern int _vips__argument_id;
 		(MIN), (MAX), (VALUE), \
 		(GParamFlags) (G_PARAM_READWRITE) );\
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -199,7 +197,7 @@ extern int _vips__argument_id;
 		(TYPE), (VALUE), \
 		(GParamFlags) (G_PARAM_READWRITE) );\
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -212,7 +210,7 @@ extern int _vips__argument_id;
 		(TYPE), (VALUE), \
 		(GParamFlags) (G_PARAM_READWRITE) );\
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -225,7 +223,7 @@ extern int _vips__argument_id;
 		(VALUE), \
 		(GParamFlags) (G_PARAM_READWRITE) ); \
 	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -236,7 +234,7 @@ extern int _vips__argument_id;
 	pspec = g_param_spec_pointer( (NAME), (LONG), (DESC), \
 		(GParamFlags) (G_PARAM_READWRITE) ); \
 	g_object_class_install_property( gobject_class,  \
-		_vips__argument_id++, pspec ); \
+		vips_argument_get_id(), pspec ); \
 	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
 		pspec, (VipsArgumentFlags) (FLAGS), (PRIORITY), (OFFSET) ); \
 }
@@ -296,6 +294,7 @@ typedef struct _VipsArgumentInstance {
  */
 typedef GHashTable VipsArgumentTable;
 
+int vips_argument_get_id( void );
 void vips__object_set_member( VipsObject *object, GParamSpec *pspec,
 	GObject **member, GObject *argument );
 typedef void *(*VipsArgumentMapFn)( VipsObject *object, GParamSpec *pspec,
@@ -303,6 +302,8 @@ typedef void *(*VipsArgumentMapFn)( VipsObject *object, GParamSpec *pspec,
 	VipsArgumentInstance *argument_instance, void *a, void *b );
 void *vips_argument_map( VipsObject *object, 
 	VipsArgumentMapFn fn, void *a, void *b );
+int vips_object_get_args( VipsObject *object, 
+	const char ***names, int **flags, int *n_args );
 typedef void *(*VipsArgumentClassMapFn)( VipsObjectClass *object_class, 
 	GParamSpec *pspec,
 	VipsArgumentClass *argument_class, void *a, void *b );
