@@ -12,8 +12,10 @@
  * 	- vips_image_write() does not ref input for non-partial images
  * 29/10/16
  * 	- add vips_image_hasalpha()
- * 11/10/1
+ * 11/10/17
  * 	- more severing for vips_image_write()
+ * 3/4/18
+ * 	- better rules for hasalpha
  */
 
 /*
@@ -2449,9 +2451,6 @@ vips_get_disc_threshold( void )
  * will default to /tmp. On Windows, vips uses GetTempPath() to find the
  * temporary directory. 
  *
- * vips uses g_mkstemp() to make the temporary filename. They generally look
- * something like "vips-12-EJKJFGH.v".
- *
  * See also: vips_image_new().
  *
  * Returns: the new #VipsImage, or %NULL on error.
@@ -2517,30 +2516,39 @@ vips_image_write_gen( VipsRegion *or,
 int
 vips_image_write( VipsImage *image, VipsImage *out )
 {
+	/* image needs to stay alive for this call. It can be unreffed during
+	 * the generate.
+	 */
+	g_object_ref( image );
+
 	if( vips_image_pio_input( image ) || 
 		vips_image_pipelinev( out, 
-			VIPS_DEMAND_STYLE_THINSTRIP, image, NULL ) )
+			VIPS_DEMAND_STYLE_THINSTRIP, image, NULL ) ) {
+		g_object_unref( image );
 		return( -1 );
+	}
 
 	if( vips_image_generate( out,
 		vips_start_one, vips_image_write_gen, vips_stop_one, 
-		image, NULL ) )
+		image, NULL ) ) {
+		g_object_unref( image );
 		return( -1 );
+	}
 
-	/* If @out is a partial image, we need to make sure that @image stays
-	 * alive as long as @out is alive.
+	/* If @out is a partial image, we need to unref @image when out is
+	 * unreffed.
 	 *
 	 * If it's not partial, perhaps a file we write to or a memory image,
 	 * we need to break any links between @image and @out created by
 	 * vips_image_pipelinev().
 	 */
 	if( vips_image_ispartial( out ) ) { 
-		g_object_ref( image );
 		vips_object_local( out, image );
 	}
 	else {
 		vips__reorder_clear( out );
 		vips__link_break_all( out );
+		g_object_unref( image );
 	}
 
 	return( 0 );
@@ -2649,7 +2657,7 @@ vips_image_write_to_buffer( VipsImage *in,
  *
  * Writes @in to memory as a simple, unformatted C-style array. 
  *
- * The caller is responsible for freeing this memory. 
+ * The caller is responsible for freeing this memory with g_free(). 
  *
  * See also: vips_image_write_to_buffer().
  *
@@ -2857,18 +2865,47 @@ vips_image_ispartial( VipsImage *image )
  * vips_image_hasalpha: (method)
  * @image: image to check
  *
- * libvips assumes an image has an alpha if it has two bands (ie. it is a
- * monochrome image with an extra band), if it has four bands (unless it's been
- * tagged as CMYK), or if it has more than four bands. 
+ * Look at an image's interpretation and see if it has extra alpha bands. For
+ * example, a 4-band #VIPS_INTERPRETATION_RGB would, but a six-band 
+ * #VIPS_INTERPRETATION_MULTIBAND would not. 
  *
  * Return %TRUE if @image has an alpha channel.
  */
 gboolean
 vips_image_hasalpha( VipsImage *image )
 {
-	return( image->Bands == 2 ||
-		(image->Bands == 4 && image->Type != VIPS_INTERPRETATION_CMYK) ||
-		image->Bands > 4 );
+	/* The result of hasalpha is used to turn on things like
+	 * premultiplication, so we are rather conservative about when we
+	 * signal this. We don't want to premultiply things that should not be
+	 * premultiplied.
+	 */
+	switch( image->Type ) { 
+	case VIPS_INTERPRETATION_B_W:
+	case VIPS_INTERPRETATION_GREY16:
+		return( image->Bands > 1 ); 
+
+	case VIPS_INTERPRETATION_RGB:
+	case VIPS_INTERPRETATION_CMC:
+	case VIPS_INTERPRETATION_LCH:
+	case VIPS_INTERPRETATION_LABS:
+	case VIPS_INTERPRETATION_sRGB:
+	case VIPS_INTERPRETATION_YXY:
+	case VIPS_INTERPRETATION_XYZ:
+	case VIPS_INTERPRETATION_LAB:
+	case VIPS_INTERPRETATION_RGB16:
+	case VIPS_INTERPRETATION_scRGB:
+	case VIPS_INTERPRETATION_HSV:
+		return( image->Bands > 3 ); 
+
+	case VIPS_INTERPRETATION_CMYK:
+		return( image->Bands > 4 ); 
+
+	default:
+		/* We can't really infer anything about bands from things like
+		 * HISTOGRAM or FOURIER.
+		 */
+		return( FALSE ); 
+	}
 }
 
 /**
@@ -3232,7 +3269,8 @@ vips_image_wio_input( VipsImage *image )
 		 */
 		if( vips_mapfile( image ) ) 
 			return( -1 );
-		image->data = image->baseaddr + image->sizeof_header;
+		image->data = (VipsPel *) image->baseaddr + 
+			image->sizeof_header;
 		image->dtype = VIPS_IMAGE_MMAPIN;
 
 		break;
