@@ -88,6 +88,7 @@
 #include <vips/thread.h>
 #include <vips/internal.h>
 #include <vips/vector.h>
+#include <vips/vips7compat.h>
 
 /* abort() on the first warning or error.
  */
@@ -511,7 +512,7 @@ vips_leak( void )
 			vips_error_buffer() );
 
 	if( vips__n_active_threads != 0 )
-		vips_buf_appendf( &buf, "threads: %d still active\n", 
+		vips_buf_appendf( &buf, "threads: %d not joined\n", 
 			vips__n_active_threads ); 
 
 	fprintf( stderr, "%s", vips_buf_all( &buf ) );
