@@ -1,6 +1,9 @@
 /* abstract base class for all vips objects
  *
  * Edited from nip's base class, 15/10/08
+ *
+ * 29/5/18
+ * 	- added vips_argument_get_id()
  */
 
 /*
@@ -209,6 +212,8 @@ static GMutex *vips__object_all_lock = NULL;
 
 static guint vips_object_signals[SIG_LAST] = { 0 };
 
+/* This has to be externally visible for compatibility with older libvipses.
+ */
 int _vips__argument_id = 1;
 
 /* Keep a cache of nickname -> GType lookups.
@@ -217,6 +222,26 @@ static GHashTable *vips__object_nickname_table = NULL;
 
 G_DEFINE_ABSTRACT_TYPE( VipsObject, vips_object, G_TYPE_OBJECT );
 
+/**
+ * vips_argument_get_id: (skip)
+ *
+ * Allocate a new property id. See g_object_class_install_property().
+ *
+ * Returns: a new property id > 0
+ */
+int 
+vips_argument_get_id( void )
+{
+	int id;
+
+	/* We probably don't need to lock: glib seems to single-thread class
+	 * creation.
+	 */
+	id = _vips__argument_id++;
+
+	return( id );
+}
+
 /* Don't call this directly, see vips_object_build().
  */
 static int
@@ -2185,6 +2210,73 @@ vips_object_find_required( VipsObject *object )
 		vips_argument_is_required, NULL, NULL ) );
 }
 
+typedef struct _VipsNameFlagsPair {
+	const char **names;
+	int *flags;
+} VipsNameFlagsPair;
+
+static void *
+vips_object_find_args( VipsObject *object,
+	GParamSpec *pspec,
+	VipsArgumentClass *argument_class,
+	VipsArgumentInstance *argument_instance,
+	void *a, void *b )
+{
+	VipsNameFlagsPair *pair = (VipsNameFlagsPair *) a;
+	int *i = (int *) b;
+
+	pair->names[*i] = g_param_spec_get_name( pspec );
+	pair->flags[*i] = (int) argument_class->flags;
+
+	*i += 1;
+
+	return( NULL );
+}
+
+/**
+ * vips_object_get_args: (skip)
+ * @object: object whose args should be retrieved
+ * @names: (transfer none) (array length=n_args) (allow-none): output array of %GParamSpec names
+ * @flags: (transfer none) (array length=n_args) (allow-none): output array of #VipsArgumentFlags
+ * @n_args: (allow-none): length of output arrays
+ *
+ * Get all %GParamSpec names and #VipsArgumentFlags for an object.
+ *
+ * This is handy for language bindings. From C, it's usually more convenient to
+ * use vips_argument_map().
+ *
+ * Returns: 0 on success, -1 on error
+ */
+int
+vips_object_get_args( VipsObject *object,
+	const char ***names, int **flags, int *n_args )
+{
+	VipsObjectClass *object_class = VIPS_OBJECT_GET_CLASS( object );
+	int n = g_slist_length( object_class->argument_table_traverse );
+
+	VipsNameFlagsPair pair;
+	int i;
+
+	pair.names = VIPS_ARRAY( object, n, const char * );
+	pair.flags = VIPS_ARRAY( object, n, int );
+	if( !pair.names ||
+		!pair.flags )
+		return( -1 );
+
+	i = 0;
+	(void) vips_argument_map( object,
+		vips_object_find_args, &pair, &i );
+
+	if( names )
+		*names = pair.names;
+	if( flags )
+		*flags = pair.flags;
+	if( n_args )
+		*n_args = n;
+
+	return( 0 );
+}
+
 /**
  * vips_object_new: (skip)
  * @type: object to create
