@@ -5,6 +5,8 @@
  * 12/8/18
  * 	- prevent float->int overflow
  * 	- a bit quicker
+ * 17/12/18
+ * 	- we were not offsetting pixel fetches by window_offset
  */
 
 /*
@@ -238,7 +240,8 @@ vips_mapim_region_minmax( VipsRegion *region, VipsRect *r, VipsRect *bounds )
 				q[z] = 0; \
 		} \
 		else \
-			interpolate( mapim->interpolate, q, ir[0], px, py ); \
+			interpolate( mapim->interpolate, q, ir[0], \
+				px + window_offset, py + window_offset ); \
 		\
 		p1 += 2; \
 		q += ps; \
@@ -260,7 +263,8 @@ vips_mapim_region_minmax( VipsRegion *region, VipsRect *r, VipsRect *bounds )
 				q[z] = 0; \
 		} \
 		else \
-			interpolate( mapim->interpolate, q, ir[0], px, py ); \
+			interpolate( mapim->interpolate, q, ir[0], \
+				px + window_offset, py + window_offset ); \
 		\
 		p1 += 2; \
 		q += ps; \
@@ -278,6 +282,8 @@ vips_mapim_gen( VipsRegion *or, void *seq, void *a, void *b, gboolean *stop )
 	const VipsImage *in = in_array[0];
 	const int window_size = 
 		vips_interpolate_get_window_size( mapim->interpolate );
+	const int window_offset = 
+		vips_interpolate_get_window_offset( mapim->interpolate );
 	const VipsInterpolateMethod interpolate = 
 		vips_interpolate_get_method( mapim->interpolate );
 	const int ps = VIPS_IMAGE_SIZEOF_PEL( in );
@@ -310,6 +316,7 @@ vips_mapim_gen( VipsRegion *or, void *seq, void *a, void *b, gboolean *stop )
 
 	/* The bounding box of that area is what we will need from @in. Add
 	 * enough for the interpolation stencil as well.
+	 * 
 	 */
 	bounds.width += window_size - 1;
 	bounds.height += window_size - 1;
