@@ -167,6 +167,7 @@ vips_resample_operation_init( void )
 	extern GType vips_quadratic_get_type( void ); 
 	extern GType vips_affine_get_type( void ); 
 	extern GType vips_similarity_get_type( void ); 
+	extern GType vips_rotate_get_type( void ); 
 	extern GType vips_resize_get_type( void ); 
 
 	vips_thumbnail_file_get_type(); 
@@ -182,6 +183,7 @@ vips_resample_operation_init( void )
 	vips_quadratic_get_type(); 
 	vips_affine_get_type(); 
 	vips_similarity_get_type(); 
+	vips_rotate_get_type(); 
 	vips_resize_get_type(); 
 }
 
